//
// diclook.c

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include "macro.h"

#define Size	256

extern int  ct, wc, cnb;
extern char aj[WordNumb][YakugoMax][JWordLen], az[WordNumb][YakugoMax][ZokuLen];
extern char egs[], pwd[], caj[YakugoMax][JWordLen], caz[YakugoMax][ZokuLen];
extern char wdw[], wdp[], wdx[], wdh[], Cmp, Dbg;
char zkc, Cap, wd[WordLen];
FILE *ejd;
extern FILE *prcs, *dbg;
//extern char *dir;

symbck(){
    int  i, j2, n1, n2;
    char mj, dums[50], zenkaku[100];

    for(i = 0; i <= YakMaxCt; i++){
      caj[i][0] = '\0';	caz[i][0] = '\0';
    }
    ct = 0;
    if(strcmp(wd, "<") == 0){
      strcpy(caj[0], "0");	ct = 1;	return;
    }
    if(strcmp(wd, ";") == 0){
      strcpy(caj[0], "0G");	strcpy(caz[0], ":");	ct = 1;	return;
    }
    if(strcmp(wd, ":") == 0){
      if(isdigit(pwd[0]) && isdigit(wdh[2])){
	j2 = ninstrc(4, wdh, ' ');
	mid(wdh, 3, j2-3, dums);
	n1 = val(pwd);	n2 = val(dums);
	if(n1 < 25 && n2 < 60){
	  strcat(aj[wc][0], "");	strcat(az[wc][0], "t");
	  aj[wc][1][0] = '\0';	caj[0][0] = '1';	caj[0][1] = '\0';
	  nhantozen(dums, zenkaku);
	  strcat(caj[0], zenkaku);	strcat(caj[0], "");
	  strcpy(caz[0], "nt");	caj[1][0] = '\0';
	  strcat(wd, " ");	strcat(wd, dums);	goto ColonEnd;
	}
      }
      strcpy(caj[0], "0F");	strcpy(caz[0], wd);
      ColonEnd:	ct = 1;	return;
    }
    if(strcmp(wd, "/") == 0){
      if(isdigit(pwd[0]) && isdigit(wdh[2])){
        if(strchr(wdh+2, ' ')){
 	  j2 = ninstrc(4, wdh, ' ');
	  mid(wdh, 3, j2-3, dums);
        }else
          strcpy(dums, wdh+2);
  if(Dbg == 'Y') fprintf(dbg, "--- symbol --- dums = %s \n", dums);
	n1 = val(pwd);	n2 = val(dums);
	if(n1 < 13 && n2 < 32){
	  strcat(aj[wc][0], "");	strcat(az[wc][0], "t");
	  aj[wc][1][0] = '\0';	caj[0][0] = '1';	caj[0][1] = '\0';
	  nhantozen(dums, zenkaku);
	  strcat(aj[wc][0], zenkaku);	strcat(aj[wc][0], "");
	  strcpy(caz[0], "nt");	caj[0][1] = '\0';
	  strcat(wd, " ");	strcat(wd, dums);	goto SlashEnd;
	}
      }
      strcpy(caj[0], "0A");	strcpy(caz[0], wd);
      SlashEnd:	ct = 1;	return;
    }
    if(strcmp(wd, "&middot;") == 0){
      strcpy(caj[0], "0E");	ct = 1;	return;
    }
    if(strcmp(wd, "\"") == 0 || strcmp(wd, "&quot;") == 0){
      strcpy(caj[0], "|h");	ct = 1;	return;
    }
    if(strcmp(wd, "\'") == 0){
      strcpy(caj[0], "|f");	strcpy(caz[0], wd);	ct = 1;	return;
    }
    if(strcmp(wd, "[") == 0 || strcmp(wd, "(") == 0){
      strcpy(caj[0], "|[");	ct = 1;	return;
    }
    if(strcmp(wd, "]") == 0 || strcmp(wd, ")") == 0){
      strcpy(caj[0], "|]");	ct = 1;	return;
    }
    if(strcmp(wd, "+") == 0){
      strcpy(caj[0], "0{");	strcpy(caz[0], wd);	ct = 1;	return;
    }
    if(strcmp(wd, "-") == 0){
      strcpy(caj[0], "0|");	strcpy(caz[0], wd);	ct = 1;	return;
    }
    if(strcmp(wd, "@") == 0){
      strcpy(caj[0], "0");	ct = 1;	return;
    }
    if(strcmp(wd, "|") == 0){
      strcpy(caj[0], "0b");	strcpy(caz[0], wd);	ct = 1;
      return;
    }
    if(strcmp(wd, ",") == 0){
      strcpy(caj[0], "0A");	strcpy(caz[0], wd);	ct = 1;
      return;
    }
    if(strcmp(wd, ".") == 0){
      strcpy(caj[0], "0B");	strcpy(caz[0], wd);	ct = 1;	return;
    }
    if(strcmp(wd, "!") == 0){
      strcpy(caj[0], "!I");	strcpy(caz[0], wd);	ct = 1;	return;
    }
    if(strcmp(wd, "?") == 0){
      strcpy(caj[0], "?H");	strcpy(caz[0], wd);	ct = 1;	return;
    }
    if(wd[0] == '%'){
      strcpy(caj[0], wd);	strcpy(caz[0], wd);	ct = 1;	return;
    }
    //if(strlen(wd) == 1 && isupper(*wd) && *wd != 'A' && *wd != 'I' && *wd != 'E'){
    //  strcpy(caj[0], "1");    strcat(caj[0], wd);
    //  strcpy(caz[0], wd);	ct = 1;	return;
    //}
    mj = wd[0];
    if(mj == '#' || mj == '*' || mj == '%'){
      caj[0][0] = mj;	caj[0][1] = '\0';	strcpy(caz[0], wd);
      ct = 1;	return;
    }
    if(isdigit(wd[0])){ digit();	return; }
}

digit(){
    int  ln, i;
    char *ptc, zenkaku[100], dums[20];
    ct = 1;
    strcpy(caz[0], "n");
    ln = strlen(wd);
    // preditɂď
    // wd̒'-'ꍇ ex 1.5-square-mile
    //if(strchr(wd, '-')){
    //  strcpy(caj[0], "1");    strcat(caj[0], wd);
    //  return;
    //}
    if(strchr(wd, '.')){
      strcpy(dums, wd);    ptc = (char*)strchr(dums, '.');
      *ptc = '\0';    nhantozen(dums, zenkaku);
      strcpy(caj[0], "1");   strcat(caj[0], zenkaku);
      strcat(caj[0], ".");
      nhantozen(ptc+1, zenkaku);    strcat(caj[0], zenkaku);
      return;
    }
    if(ln > 2 && (strcmp(wd+ln-2, "th") == 0 || strcmp(wd+ln-2, "st") == 0
    || strcmp(wd+ln-2, "nd") == 0 || strcmp(wd+ln-2, "rd") == 0)){
 if(Dbg == 'Y') fprintf(dbg, "--- digit --- wd = %s \n", wd);
      strcpy(dums, wd);
      *(dums+ln-2) = '\0';	nhantozen(dums, zenkaku);
      *caj[0] = '\0';
/*	strink(caj[0], "5", zenkaku, "Ԗڂ",(char*)0);*/
      strink(caj[0], "5", zenkaku,(char*)0);
      strcat(caz[0], "R");
      return;
    }
    if(ln > 2 && (strcmp(wd+ln-1, "M") == 0)){
      strcpy(dums, wd);
      *(dums+ln-1) = '\0';	nhantozen(dums, zenkaku);
      *caj[0] = '\0';
      strink(caj[0], "1", zenkaku, "OO", (char*)0);
      strcat(caz[0], "R");
      return;
    }
    strcpy(caj[0], "1");	strcpy(caz[0], "nt");
    if(ln > 2 && strcmp(wd+ln-2, "0s") == 0){ strcpy(dums, wd);
      *(dums+ln-1) = '\0';	nhantozen(dums, zenkaku);
      strcat(caj[0], zenkaku);	strcat(caj[0], "N");
      return;
    }
    ptc = wd;
    while(*(++ptc)){
      if((! isdigit(*ptc)) && *ptc != ',') return;
    }
    if((strcmp(pwd, "in") == 0 || strcmp(pwd, "of") == 0 || strcmp(pwd, "by") == 0) && strlen(wd) == 4){
      nhantozen(wd, zenkaku);	strcat(caj[0], zenkaku);
      strcat(caj[0], "N");	return;
    }
    // Sept. 9 -> XX
    if(instr(aj[wc][0], "") && !strstr(aj[wc][0], "j") ){
      if(val(wd) < 32){
	strcpy(caj[0],aj[wc][0]);
	nhantozen(wd, zenkaku);	strcat(caj[0], zenkaku);
	strcat(caj[0], "");
	//	aj[wc][0][0] = '\0';
	strcat(caz[0], az[wc][0]);
	wc--;	return;
      }
      if(ln == 4){
	nhantozen(wd, zenkaku);	strcat(caj[0], zenkaku);
	strcat(caj[0], "N");	return;
      }
    }
    nhantozen(wd, zenkaku);	strcat(caj[0], zenkaku);
    strcpy(caz[0], "n");	strcpy(caz[1], "n");
    strcpy(caj[1], caj[0]);	caj[1][0] = '5';	cnb++;
    zkc = '\0';
}

ptcal(dics, dic,wdh)
char dics[26][27], wdh[];
int  dic[26];
{
    int  i, p0, p1, ppt;
    char r3, *ptc;
    p0 = wd[0];
    if(p0 < 92) ppt = p0-65;
    else ppt = p0-97;
    if(ppt < 0 || ppt > 25){
      printf("\n Pointer error  ppt = %d wdh = %s \n", ppt, wdh);
//      return(0);
      if(strlen(wdh) == 0) return(0);
      ppt = 0;
    }
    if(strlen(wd) == 1){ i = 0;	goto PTA; }
    p1 = wd[1];
    if(p1 < 65){ i = 0;	goto PTA; }
    if(p1 < 92) p1 += 32;
    i = 0;
    while(dics[ppt][i] != p1){
      if(dics[ppt][i] > p1){ i--;	break; }
      if(dics[ppt][i] == ' ' || dics[ppt][i] == '\0'){
	i = 0;	goto PTA;
      }
      else i++;
    }
    i++;
    PTA:
    ppt = dic[ppt]+i+1;
    if(strlen(wd) == 1) return(ppt);
    if(ppt == 23){
      if(wd[2] == '\0' || wd[2] == ' '){
/*	if(wdh[2] == ' ') r3 = wdh[3];
	else{ ptc = (char*)index(wdh,' ');	r3 = *(++ptc); }*/
/*	if(wd[2] == '\0') r3 = 'z';
	else r3 = wd[3];
	if((r3 >= 'o' && r3 <= 'z') || (r3 >= 'O' && r3 <= 'Z')) ppt += 2;
	else ppt++;*/
        ppt++;
      }
      return(ppt);
    }
    if(ppt == 38 && wd[2] >= 'n'){ ppt++;	return(ppt); }
    if(ppt == 87 && wd[2] >= 'v'){
      if(strcmp(wd+1, "ave") == 0 && (ptc = (char*)index(wdh,' ')) &&
	*(++ptc) == 'a' && *(++ptc) == ' ') ppt++;
      else ppt += 2;
      return(ppt);
    }
    if(ppt == 101){
      if(wd[2] != '\0' && wd[2] != ' '){ ppt += 2;	return(ppt); }
      else{
	r3 = tolower(wdh[3]);
	if(r3 > 'o'){ ppt++;	return(ppt); }
      }
    }
    if(ppt == 127 && wd[2] >= 'l'){ ppt++;	return(ppt); }
    if(ppt == 202 && wd[2] >= 'i') ppt++;
    return(ppt);
}


diclook(ppt, cs)
int  ppt, *cs;
{
    int  ln, lnw, nx, j, yn;
    char ejr[Size], ejw[170], egw[WordLen], jpw[JWordLen], zkw[33];
    char rwc, *pts, *jw, *jt;
    char wdw2[SentLen];

    nx = 1;
    while(nx > 0){
      dictread(ppt, ejr, &nx);
      ct = 0;
      while(j = instr(ejr, wd)){
	decompose(ejr, j, ejw);
	if(ct == 9){ ct = 0;	continue; }
	jw = (char*)index(ejw, ';');
	ptcpy(egw, ejw, jw);

	// add_debug(ejw, nx);

	if(strlen(wdx) > strlen(egw)) goto NTT;
	jt = (char*)index(jw+1, ';');
	ptcpy(jpw, jw+1, jt);
	ln = strlen(jt+1);
	ptcpy(zkw, jt+1, jt+ln);
	if(zkc){
	  codeCk(jpw, egw);
	  if(ct == -1){ ct = 0;	goto NTT; }
	}
	lnw = strlen(egw)-1;
	if(instr(wdw, egw) == 1 && wdw[lnw+1] == ' ') goto OKYAKU;
	if(pts = (char*)index(egw, '#')){
	  umeJukugo(pts, egw, wdw, &yn);
	  if(yn == 0) goto NTT;
	  else goto OKYAKU;
	}
	tailConj(lnw,egw,wdw,wdp,cs);
// if(Dbg == 'Y') fprintf(dbg, "--- diclook --- ct = %d \n", ct);
 //	if(ct == -1){ ct = 0;	goto NTT; }

	if(wdw != (char*)strstr(wdw, egw)) {
	  strcpy(wdw2, wdw);
	  StrnToLower(wdw, strlen(egw));
	  if(wdw != (char*)strstr(wdw, egw)){
	    ct = 0; strcpy(wdw, wdw2); goto NTT; }
	//	if(wdw != (char*)strstr(wdw, egw)) goto NTT;
	}
	rwc = wdw[strlen(egw)];
	if(rwc != ' ' && rwc != '\0' && rwc != '-' && rwc != '.' && zkc != 'h')
	  goto NTT;

	OKYAKU:
 if(Dbg == 'Y') fprintf(dbg, "--- diclook --- ejw = %s \n", ejw); // 
	yakSaiyo(jpw,egw,zkw,wdx,wdp);

	NTT:
	cutoff(ejr, 0, strlen(ejw));
      }
      if(nx > 245 && nx < MaxNr && ppt < nx) ppt = nx;
    }
/* printf("--- diclook1 --- nx = %d  wdx = %s \n", nx,wdx);*/
}

dictread(ppt, ejr, nx)
int  ppt, *nx;
char ejr[];
{
    long ptl;
    char *ptc;
    int ret;
    ptl = (long)(ppt-1)* Size;
    rewind(ejd);
    if(fseek(ejd, ptl, SEEK_SET)){    // 0 -> SEEK_SET
      printf("seek error\n");	exit(1);
    }
    ret = fread(ejr, Size-4,1, ejd);
    if(ret != 1){
      return(1); // error
    }
    fscanf(ejd, "%4d", nx);
    *(ejr+ Size-5) = '\0';
    if(ptc = (char*)rindex(ejr, ':')) *(ptc+1) = '\0';
    else{
      if(*ejr && *ejr != ' '){
	printf("Abnomal   ejr = %s \n", ejr);
	exit(1);
      }
    }
}

decompose(ejr, j, ejw)
int  j;
char ejr[], ejw[];
{
    int  i, ln;
    ln = strlen(wd);
    if(ejr[j+ln-1] != ' ' && ejr[j+ln-1] != ';' && ejr[j+ln-1] != '-'){
      cutoff(ejr, 0,j);	ct = 9;	return;
    }
    if(j > 1 && ejr[j-2] != ':'){ cutoff(ejr, 0, j);	ct = 9;	return; }
    if(j > 1) cutoff(ejr, 0, j-1);
    i = 0;
    while(ejr[i] != ':'){
      ejw[i] = ejr[i];	i++;
      if(i >= JWordLen*2+WordLen+ZokuLen){
	ejw[i] = '\0';
	printf("Too long!! \n");	exit(1);
      }
    }
    ejw[i++] = ':';	ejw[i] = '\0';
}

yakSaiyo(jpw, egw, zkw, wdx, wdp)
char jpw[], egw[], zkw[], wdx[], wdp[];
{
    int  lnw, pp;
    lnw = strlen(egw);
    if(lnw ==strlen(wdx) || (wdx[0] =='\0' && cnb > -1 && (!index(egw,' ')))){
      cnb++;
      if(cnb > YakMaxCt){
	printf("pP\'%s\' ̖̐܂B\n", wd);
	for(pp = 0; pp < cnb; pp++)
	  printf("caj[%d] = %s \n", pp, caj[pp]);
	printf("o^ȂĂ");
	exit(1);
      }
      if(wdx[0] == '\0') strcpy(wdx, egw);
      strcpy(caj[cnb], jpw);
      if(zkc != 'h'){
	caz[cnb][0] = zkc;	caz[cnb][1] = '\0';	strcat(caz[cnb], zkw);
      }
      else strcpy(caz[cnb], zkw);
      return;
    }
    strcpy(wdx, egw);
    strcpy(caj[0], jpw);
    if(zkc != 'h'){
      caz[0][0] = zkc;	caz[0][1] = '\0';	strcat(caz[0], zkw);
    }
    else strcpy(caz[0], zkw);
    cnb = 0;
    if((!index(egw, '#')) && zkc != 's'){
      strcpy(wdp, egw);	strcpy(wd, egw);
    }
}

Conj(wdp, wdh, cs)
char wdp[WordLen], wdh[SentLen];
int  *cs;
{
    int  i, lnd, ln;
    char dums[8], rc, *rpt;
    static char zok[5] = {'p', 'x', 'C', 'U', 'G'},
	 ied[4][5] = {"ied", "ies", "ier", "iest"},
	 ed[4][4] = {"ed", "es", "er", "est"},
	 d[5][4] = {"d", "s", "r", "st", "ing"};

    if(zkc == 's') goto CONJ;
    preConj(cs, wdp, wdh);
    if(ct == -1){ ct = 0;	return; }
    CONJ:	if(strlen(wd) < 4) goto DEFLT;
    ln = strlen(wd);
//    if(strstr(wd, "'s")){ wd[ln-2] = '\0';	zkc = 'h';	return; }
    if(strcmp(wd+ln-2, "'s") == 0){ wd[ln-2] = '\0';	zkc = 'h';	return; }
    switch(*cs){
    case 1:
      *cs = 2;
//  fprintf(dbg, "--- Conj --- wd = %s \n", wd);
    ln = strlen(wd);
    right(wd, dums, 4);
    if(strcmp(dums+1, "ves") == 0 && strcmp(wd, "believes") != 0 && strcmp(wd, "leaves")){
      if(*dums == 'i')
        ptreplace(wd+ln-3, 3, "fe");
      else
        ptreplace(wd+ln-3, 3, "f");
      zkc = 'x';
      return;
    }
    for(i = 0; i <= 3; i++){
      lnd = strlen(ied[i]);	rpt = wd+ln-lnd;
      if(strcmp(rpt, ied[i]) == 0){
	strcpy(rpt, "y");	zkc = zok[i];	caj[0][0] = '3';	return;
      }
    }
    case 2:
      *cs = 3;	ln = strlen(wd);
    if(ln >= 4){
      for(i = 0; i <= 3; i++){
	lnd = strlen(ed[i]);	rpt = wd+ln-lnd-1;
	ptcpy(dums, rpt-1, rpt+1);
        if(!strcmp(wd, "copes") || !strcmp(wd, "tapes")) continue;
	if(*rpt == 'e') continue;
  if(Dbg == 'Y') fprintf(dbg, "--- Conj cs=3 ---  wd = %s dums = %s *rpt = %c \n", wd, dums, *rpt);
        if(!strcmp(wd, "propelled") || !strcmp(wd, "cancelled")
	// || strcmp(wd+strlen(wd)-4, "lled") == 0    // annulled compelled
	|| !strcmp(wd, "compelled")
        || !strcmp(wd, "repelled")  || !strcmp(wd, "annulled")){
          *rpt = '\0';
	  zkc = 'p';	caj[0][0] = '3';	return;
        }
        // isVowel:ꉹ̔
	if(ln > 4 && isVowel(*(rpt-1)) && (! isVowel(*rpt)) &&
	(! isVowel(*(rpt-2))) && strcmp(wd, "anchored") && strcmp(wd, "stupider") ){
	  // focuses canseledłȂ
          if(strcmp(dums, "en") && strcmp(dums, "it") && strcmp(dums, "er") 
          && *rpt != 'y' && *rpt != 'w' && *rpt != 'x' && *rpt != 'p' 
	     && *rpt != 'm'    // blossomed
          && *rpt != 's' && *rpt != 'l' && *rpt != 't') continue;
	}
	if(strcmp(++rpt, ed[i]) == 0){
	  if(i == 0 && wd == (char*)strstr(wd, "be ")) continue;
	  *rpt-- = '\0';
// tattooed̏ꍇǉ
          if(strcmp(dums, "oo") == 0){
            zkc = zok[i];	caj[0][0] = '3';	return;
          }
//
	  if(i != 1 && i != 4){
	    ln -= (++lnd);	rc = *rpt--;
	    if(ln >= 3 && rc == *rpt && rc != 's' && rc != 'l' && rc != 'f')
	      *(++rpt) = '\0';
	  }
	  zkc = zok[i];	caj[0][0] = '3';	return;
	}
      }
    }
    case 3:
      *cs = 4;	ln = strlen(wd);
    if(! strstr(wd, "'s") && (zkc != 'h' || Cap != 'O')){
      for(i = 0; i <= 4; i++){
	lnd = strlen(d[i]);	rpt = wd+ln-lnd;
	if(strcmp(rpt, d[i]) == 0){
	  if(i == 0 && wd == (char *)strstr(wd, "be ")) continue;
          if(i == 0 && *(rpt-1) == 'o') continue;
          if(strcmp(wd+ln-2, "ss") == 0 && strcmp(d[i], "s") == 0) continue;
          if(strcmp(wd, "hoping") == 0) continue;
	  *rpt-- = '\0';	ln -= (++lnd);
	  if(i == 4){
	    rc =*rpt--;
	    if(ln >= 3 && rc == *rpt && rc != 's' && rc != 'l' &&
	      rc != 'f' && rc != 'e') *(++rpt) = '\0';
	  }
	  zkc = zok[i];	caj[0][0] = '3';	return;
	}
      }
    }
    case 4:
      *cs = 5;
      lnd = strlen(wd);	rpt = wd+lnd-3;
    if(strcmp(rpt, "ing") == 0){
      if(strncmp(rpt-2, "ll", 2) == 0) *(rpt-1) = '\0';
      if(*(rpt-1) == 'y') *(rpt-1) = 'i'; 
      strcpy(rpt, "e");	zkc = 'G';	caj[0][0] = '3';	return;
    }
    case 5:
      *cs = 6;
    if(!index(wd, ' ')){
      lnd = strlen(wd)-1;
      if(ninstr(lnd, wd, "th") == lnd){
	cutoff(wd, --lnd, 2);	zkc = 'n';	caj[0][0] = '1';	return;
      }
    }
    case 6:	*cs = 7;
      lnd = strlen(wd)-2;
      if(lnd > 2 && instr(wd, "men") == lnd){
	replaces(wd, lnd, 1, "a");	zkc = 'x';	return;
      }
    case 7:
      *cs = 8;
      if(! index(wd, ' ')){
	setsubiGo();	if(zkc == 'l'){ caj[0][0] = '5';	return; }
      }
    default:
      DEFLT:	*cs = 9;
      conjDflt(wdp, wdh);
    }
}

conjDflt(wdp, wdh)
char wdp[WordLen], wdh[SentLen];
{
    int  ln;
    char *gobi;
 if(Dbg == 'Y') fprintf(dbg, "--- conjDflt --- wd = %s \n", wd);
    ln = strlen(wd);
    if(ln >= 2) gobi = wd+ln-2;
    else gobi = wd;
    if(strcmp(gobi, "'s") == 0 || strcmp(gobi, "s'") == 0){
      if(strcmp(gobi-2, "ies'") == 0){
        strcpy(gobi-2, "y"); zkc = 'h';
        return;
      }
      left(wd,wdp,strlen(wdp)-2);
      if(zkc == 'O' || wc == -1){
	if(*wd > 90) *wd -= 32;
	strcpy(wdp, wd);	zkc = 'h';
      }
      else zkc = 's';
      *gobi = '\0';	return;
    }
    else {
      if(zkc == 'O' || zkc == 'h' || Cap == 'O'){
	if(*wd > 90) *wd -= 32;
	proper();	return;
      }
      if(Cap == 'O' && Cmp == '~'){
	if(*wdh > 90) *wdh -= 32;
	proper();	return;
      }
      if(Cmp == '~'){ caj[0][0] = '1';	caj[0][1] = '\0';
	strcat(caj[0], wd);	caj[1][0] = '\0';	cnb++;
	strcpy(wdx, wd);	strcpy(wdw, wdh);	return;
      }
      if(zkc == 's'){
 if(Dbg == 'Y') fprintf(dbg, "--- conjDflt --- zkc = %c \n", zkc);
	strcpy(caj[0],"1");	strcat(caj[0],wd);	caj[1][0] = '\0';
	return;
      }
      if(wc == -1 && zkc == 'x'){
      }
    }
    if(zkc != '~') zkc = '\0';
}

engcopy(){
    int  i, ln;
    char *setubi[3] = {"tional","ental","ical"};
  if(Dbg == 'Y'){ fputs("caj[i], caz[i] ɃRs[B", dbg);
      if(Dbg == 'Y') fprintf(dbg, "wd = %s \n", wd);
    }
    strcpy(caj[0], "1");	strcpy(caz[0], "Y");
    strcpy(caj[1], "3");	strcpy(caz[1], "i");
    strcpy(caj[2], "6");	caz[2][0] = '\0';
    for(i = 0; i <= 2; i++) strcat(caj[i], wd);
    strcat(caj[1], "");
    // internet address̏ꍇÂ
    if(strchr(wd, '.') && (strchr(wd, '@') || strchr(wd, '/')))
      caj[1][0] = '\0';
    caj[3][0] = '\0';
    ln = strlen(wd);
    for(i = 0; i <= 2; i++){
      if(ln > strlen(setubi[i]) && strcmp(wd+ln-strlen(setubi[i]), setubi[i]) == 0){
        caj[2][0] = '5';
        strcat(caj[2], "");
        return;
      }
    }
    strcat(caj[2], "");
}

// ŗL
proper(){
    caj[0][0] = '1';	caj[0][1] = '\0';	strcat(caj[0], wd);
    caz[0][0] = 'K';	caz[0][1] = '\0';	caj[1][0] = '\0';
    if(zkc != 'h') zkc = 'K';
}

// ߍ݌^n̏
//
umeJukugo(pts, egw, wdw, yn)
int  *yn;
char *pts, egw[45], wdw[100];
{
    int  j2;
    char eg1[40], eg2[40], eg3[40], *ps2;
    ptcpy(eg1, egw, pts-1);        // eg1 = egw̒#O̕
    if(*(pts+1) == ' ') strcpy(eg2, pts+2);
    else strcpy(eg2, pts+3);
    if(wdw != (char*)strstr(wdw, eg1) || *(wdw+strlen(eg1)) != ' '){
      *yn = 0;	return;
    }
    if(ps2 = (char*)index(eg2, '#')){
      if(*(ps2+1) == ' ') strcpy(eg3, ps2+2);
      else strcpy(eg3, ps2+3);
      *(ps2-1) = '\0';
    }
 if(Dbg == 'Y') fprintf(dbg, "--- umeJukugo --- %s %s %s \n", wdw, eg1, eg2);
    umeck(wdw, eg1, eg2, pts, 1, &j2, yn);
    if(*yn == 0) return;
    if(ps2) umeck(wdw, eg2, eg3, ps2, j2, &j2, yn);
}

umeck(wdw, ed, e2, ps, l0, j2, yn)
int  l0, *j2, *yn;
char wdw[100], ed[40], e2[40], *ps;
{
    int  l1, jc, jp, jq, ll, sp;
    char z, rm[50];
    l1 = strlen(ed)+l0;
    while((char) 1){
      *j2 = ninstr(l1, wdw, e2);
//  fprintf(prcs, "--- umeck --- wdw = %s ed = %s e2 = %s ps = %s j2 = %d \n", wdw, ed, e2, ps, *j2);
//  fprintf(prcs, "--- umeck --- l0 = %d l1 = %d ll = %d ps = %s \n", l0, l1, ll, ps);
      if(*j2 == 0 || (*(ps+1) != ' ' && *j2 == l1+1)){
	*yn = 0;	return;
      }
      if(wdw[*j2-2] == ' '){
	jc = ninstr(l1, wdw, " , ");
	jp = ninstr(l1, wdw, " . ");
	jq = ninstrc(l1, wdw, '?');
	if((jc > 0 && jc < *j2) || (jp > 0 && jp < *j2) || (jq > 0 && jq < *j2)){
	  *yn = 0;	return;
	}
	jp = l1;
	sp = 0;
	while(jp = ninstrc(++jp, wdw, ' ')){
	  if(jp < *j2){
	    sp++;
	    if(sp > 5){ *yn = 0;	return; }
	  }
	  else break;
	}
        ll = *j2 + strlen(e2);
//  fprintf(prcs, "--- umeck --- strlen(wdw) = %d \n", strlen(wdw));
        if((strlen(wdw) != ll-1 || (ps > 0 && strcmp(ed, e2) != 0)) &&
        wdw[ll-1] != ' '){
//  fprintf(prcs, "--- umeck --- strlen(wdw) = %d \n", strlen(wdw));
          if(wdw[ll-1] == '\0') break;    // ̂Psǉ
	  if(wdw[ll-1] == '.' && strstr(wdw, "h") && isupper(wdw[ll+1])){
	    insertc(wdw, ll-1, ' ', strlen(wdw));	break;
	  }
	  else l1 = ll;
        }
	else break;
      }
      else l1 = *j2+1;
    }
    if(*(ps+1) == ' '){
      *yn = 1;	return;
    }
    z = *(ps+1);
    mid(wdw, l1+1, *j2-l1-2, rm);
    if((z == '2' || z == '8') && *rm && index(rm, ' ')){
      *yn = 0;	return;
    }
    if(isdigit(z) && *rm && z != '8'){ *yn = 1;	return; }
    pronounCk(z, rm, yn);
 if(Dbg == 'Y') fprintf(dbg, "--- umeck --- >%s %s \n", ps, rm);
}

codeCk(jpw, egw)
char jpw[], egw[];
{
    char code, dums[5];
    ct = -1;	code = jpw[0];
    if((zkc == 'p' || zkc == 'G') && code != '3' && code != '4') return;
    if((zkc == 'C' || zkc == 'U') && code != '5' && code != '6') return;
    if(zkc == 'x' && code != '1' && code != '3' && code != '4'){
      if(code != '5' || strlen(egw) < 6) return;
      else{
	right(egw, dums, 3);
	if(strcmp(dums, " of") != 0) return;
      }
    }
    if(zkc == 'l' && code != '5') return;
    ct = 0;
}

pronounCk(z, rm, yn)
int *yn;
char z, rm[50];
{
    int  ln;
    if(z == '8' && (strcmp(rm, "a")==0 || strcmp(rm, "an")==0 ||
    strcmp(rm, "the")==0 || strcmp(rm, "this")==0 || strcmp(rm, "that")==0)){
      *yn = 1;	return;
    }
    if(z == 'h' && (strcmp(rm, "me")==0 || strcmp(rm, "you")==0 ||
    strcmp(rm, "him")==0 || strcmp(rm, "her")==0 || strcmp(rm, "us")==0 ||
    strcmp(rm, "them")==0)){
      *yn = 1;	return;
    }
    ln = strlen(rm);
    if(z == 's' && (instr(rm, "self")==ln-3 || instr(rm, "selves")==ln-5)){
      *yn = 1;	return;
    }
    if(z == 'D' && (strcmp(rm, "it")==0 || strcmp(rm, "this")==0 ||
    strcmp(rm, "that")==0 || strcmp(rm, "them")==0 || strcmp(rm, "these")==0 ||
    strcmp(rm, "those")==0)){
      *yn = 1;	return;
    }
    if(z == 'p' || z == '8'){
      if(strcmp(rm, "my")==0 || strcmp(rm, "your")==0 || strcmp(rm, "his")==0 ||
      strcmp(rm, "her")==0 || strcmp(rm, "its")==0 || strcmp(rm, "our")==0 ||
      strcmp(rm, "their")==0){
        *yn = 1;	return;
      }
      if(ln >= 3 &&
      (ninstr(ln, rm, "'s")==ln-1 || ninstr(ln-1,rm, "s'")==ln-1)){
	*yn = 1;	return;
      }
    }
    *yn = 0;
}

tailConj(lnw, egw,wdw, wdp, cs)
int  lnw, *cs;
char egw[WordLen], wdw[SentLen], wdp[];
{
    static char self[8][11] = {"myself","yourself","himself","herself","itself",
	"ourselves","yourselves","themselves"};
    int  i, sp, sp1, jo, js;
    char wdn[SentLen];

    sp = 0;	lnw++;
    while(sp1 = ninstrc(sp+1, egw, ' ')) sp = sp1;
 if(Dbg == 'Y') fprintf(dbg, "--- tailConj --- sp = %d egw = %s  wdw = %s \n", sp, egw, wdw);

    if(sp && strcmp(egw, wdw)){
      if(strstr(egw,"oneself") && (strstr(wdw,"self") || strstr(wdw,"selves"))){
	jo = instr(egw, "oneself");
	for(i = 0; i < 8; i++){
	  js = instr(wdw, self[i]);
	  if(js){ replaces(egw, jo-1, 7, self[i]);	break; }
	}
	goto ENDSP;
      }
      left(wdn, egw, sp);
      if(*wdn == *wdw+32) *wdw += 32;
 if(Dbg == 'Y') fprintf(dbg, "--- tailConj --- wdn = %s \n", wdn);
      if(strstr(wdw, wdn)) jukugoLook(sp, lnw, cs, egw);
      else ct = -1;
      ENDSP:;
    }
}

// ňpĂ݂āA`FbN ϐct
//
jukugoLook(sp, lnw, cs, egw)
int  sp, lnw, *cs;
char egw[];
{
    int  ksp, jju, kju;
    char wdn[SentLen], dumrn[WordLen], zkcpresv;
    ksp = lnw-3;
    if(ksp < sp+2) ksp = sp+2;
    if((pwd[0] != '#' && lnw >= strlen(wdw)-1) ||
    (pwd[0] == '#' && lnw >= strlen(wdw)+1)){
      ct = -1;	return;
    }
    jju = ninstrc(lnw, wdw, ' ');
    if(jju == 0) jju = strlen(wdw);
    else jju--;
    left(wdn, wdw, jju);
    if(ninstrc(ksp, wdn, ' ')){
      ct = -1;	return;
    }
    if(lnw > strlen(wdn)){
      ct = -1;	return;
    }
    if(lnw != strlen(wdn) &&
    (instr(wdn, egw) != 1 || wdn[lnw] != '.' || lnw != strlen(wdn)-1)){
      left(dumrn, wdn, ksp);	kju = instr(egw, dumrn);
      if(kju != 1){ ct = -1;	return; }
      strcpy(wdp, wd);
      if(strlen(wdn) >= WordLen)
	printf("\n Too long word [%s] \n", wdn);
      if(strstr(wdn, "'s")) return;  // ň's̎

      strcpy(wd, wdn);	zkcpresv = zkc;	zkc = 'j';
      do{
 if(Dbg == 'Y') fprintf(dbg, "--- jukugoLook --- wdp = %s  wdn = %s \n", wdp,wdn);
	Conj(wdn, wdw, cs);
 if(Dbg == 'Y') fprintf(dbg, "--- jukugoLook --- wd = %s  egw = %s \n", wd,egw);
	if(strcmp(wd, egw) == 0){
	  right(egw, dumrn, lnw-sp);	replaces(wdw, sp, jju-sp, dumrn);
	  strcpy(wdp, wdn);	break;
	}
	else{
	  strcpy(wd, wdn);
	  if(*cs == 9){
	    zkc = '\0';
// oO艞}:̂PC
//	    if(caj[0][0]) caj[0][0] = '\0';
	  }
	}
      } while(zkc);
      strcpy(wd, wdp);
      if((!zkc) && zkcpresv) zkc = zkcpresv;
      *cs = 1;
    }
}

preConj(cs, wdp, wdh)
int  *cs;
char wdp[], wdh[];
{
    int  i, mj, ln, spc, jh;
    char kakt, *gobi;
    static char ryaku0[4][5] = {"'re","n't","'ll","'ve"},
	sei0[4][7] = {" are"," not"," will"," have"},
	ryaku[19][9] = {"I'm","She's","she's","He's","he's","It's","it's","That's",
	"that's","Who's","who's","What's","what's","How's","There's","there's","Where's","Here's","here's"},
	sei[19][10] = {"I am","She is","she is","He is","he is","It is","it is","That is",
	"that is","Who is","who is","What is","what is","How is","There is","there is","Where is","Here is","here is"};
    ct = 0;
    if(zkc == 'j') goto IRRG;
    mj = wd[0];
    if(*cs >= 2 || zkc == 'p' || zkc == 'P'){
      if(mj >= 65 && mj <=90 && (mj != 'I' || wd[1] != ' ')) goto CAPT;
      else { strcpy(wd, wdp);	return; }
    }
    else strcpy(wdp, wd);
    ln = strlen(wd);
    if(strstr(wdh, "\'")){
    }
    if(strchr(wd, '\'')){
      if(strcmp(wd, "Won't") == 0 || strcmp(wd, "won't") == 0){
	strcpy(wd+1, "ill");	ptreplace(wdh+1, 4, "ill not");
	zkc = 'Y';	ct = -1;	return;
      }
      if(strcmp(wd, "can't") == 0 || strcmp(wd, "Can't") == 0){
	strcpy(wd+2, "n");	ptreplace(wdh+2, 3, "n not");
        zkc = 'Y';	ct = -1;	return;
      }
      if(strstr(wdh+ln-2, "'d better")){
	*(wd+ln-2) = '\0';	ptreplace(wdh+ln-2, 2, " had");
	zkc = 'Y';	ct = -1;	return;
      }
      if(strstr(wdh+ln-2, "'d like") || strstr(wdh+ln-2, "'d be")){
	*(wd+ln-2) = '\0';	ptreplace(wdh+ln-2, 2, " would");
	zkc = 'Y';	ct = -1;	return;
      }
      for(i = 0; i <= 3; i++)
	if(jh = instr(wd, ryaku0[i])){
	  cutoff(wd, jh-1, 3);
	  replaces(wdh, jh-1, 3, sei0[i]);
	  zkc = 'Y';	ct = -1;	return;
        }
      for(i = 0; i <= 18; i++)
	if(strcmp(wd, ryaku[i]) == 0){
	  spc = instrc(sei[i], ' ');
	  left(wd, sei[i], spc-1);	jh = instr(wdh, ryaku[i]);
	  replaces(wdh, jh-1, strlen(ryaku[i]), sei[i]);
	  zkc = 'Y';	ct = -1;	return;
        }
    }
    if(zkc == 'K' && (! index(wd, ' ')) && *cs > 2){
      *wd -= 32;
      if(strcmp(wd, wdh) || *pwd == '#'){
	proper();	strcpy(wdp, wd);	ct = -1;	return;
      }
      else *wd += 32;
    }
    if(*(wdp+1)) gobi = wdp+strlen(wdp)-2;
    CAPT:
    // 啶Ŏn܂P̏
    if(mj >= 65 && mj <= 90){

      // 啶ŕ(s)̎@@Ȃ̂Œ
      // Saudis ̂悤Ɂi`lj\L͎ɓo^
      // wd = 'Z'̂悤1̏ꍇ
      if(strlen(wd) == 1){
        zkc = 'K';
        ct = 0; return;
      }

      if(strlen(wd) > 2){
        if(*(wd+1)) gobi = wd+strlen(wd)-2;
        if(strcmp(gobi, "s'") == 0){
          // Stradivarius' ̏ꍇ
          *(gobi+1) = '\0';
          zkc = 'h'; ct = -1; return;
        }
        if(strcmp(gobi, "'s") == 0){
          // Israel's ̏ꍇ
  	  *gobi = '\0';
          zkc = 'h';
          ct = -1; return;
        }
      }

      if(strlen(wd) > 2 && wc != -1){
        // Chenges
        if(*(wd+1)) gobi = wd+strlen(wd)-1;
        if(strncmp(gobi-2, "ge", 2) == 0 && strcmp(gobi, "s") == 0){
          *gobi = '\0'; *wd += 32; zkc = 'K'; ct = -1; return;
        }
        // Discusses
        if(*(wd+1)) gobi = wd+strlen(wd)-2;
        if(strncmp(gobi-2, "ss", 2) == 0 && strcmp(gobi, "es") == 0){
          *gobi = '\0'; *wd += 32; zkc = 'K'; ct = -1; return;
        }
        // Stories
        gobi = wd+strlen(wd)-3;
        if(strcmp(gobi, "ies") == 0){
          strcpy(gobi, "y"); *wd += 32; zkc = 'K'; ct = -1; return;
        }
      }

      if(strlen(wd) > 1){
        // Peports ̂悤ȌóH
        gobi = wd+strlen(wd)-1;
        if(wc != -1 && strcmp(gobi-1, "ss") && strcmp(gobi-2, "ics") 
        && strcmp(gobi-2, "ies") && strcmp(gobi-1, "is")
        && strcmp(gobi-1, "us") && strcmp(gobi, "s") == 0){
          // Robotics i`Hwĵ悤ȏꍇ
          *gobi = '\0'; *wd += 32; zkc = 'K'; ct = -1; return;
          // Oscars iŗLĵ悤ȏꍇ@
        }
      }

      if(wc == -1 || pwd[0] == '.' || instr(pwd,"..") > 0 || pwd[0] == '?' ||
      pwd[0] == '!' || pwd[0] == '%' || pwd[0] == '\"' || pwd[0] == '\''){
  if(Dbg == 'Y') fprintf(dbg, "--- preConj [?] --- wd = %s  zkc = %c  pwd[0] = %c \n", wd, zkc, pwd[0]);
  	*wd += 32;
        for(i = 0; i < strlen(wd); i++){
          if(isupper(wd[i])) wd[i] = tolower(wd[i]);
        }
	if(zkc != 'h') zkc = 'O';
	if(*cs >= 3) return;
        // Tuesday's weatheȑꍇ
	if(strcmp(gobi, "'s") == 0 || strcmp(gobi, "s'") == 0){
	  ptcpy(wd, wdp, gobi);
          zkc = 'h';
//          strcpy(caj[0]+1, wd);    // 
	}
      }
      else {
	if(strcmp(gobi, "'s") == 0 || strcmp(gobi, "s'") == 0){
	  ptcpy(wd, wdp, gobi);
          zkc = 'h';
//          strcpy(caj[0]+1, wd);    // 
	}
	else {
	  if(strcmp(--gobi, "ans") == 0){
	    ptcpy(wd, wdp, gobi+2);	zkc = 'x';
	  }
          else if(strchr(wd, '-')){ 
            // wd̓啶'-'܂ގ
            for(i = 0; i < strlen(wd); i++){
              if(isupper(wd[i])) wd[i] = tolower(wd[i]);
            }
            jh = instrc(wd, '-');
            if(jh >= 2 && jh <= strlen(wd)-2){
              // wd[jh-1]strlen(wd)-jh+1폜
              cutoff(wd,jh-1,strlen(wd)-jh+1);	strcpy(wdp,wd);
              zkc = Cmp = '~';	ct = -1;	return;
            }
          }
	  else {
  if(Dbg == 'Y') fprintf(dbg, "--- preConj [s] --- wd = %s  zkc = %c  pwd[0] = %c \n", wd, zkc, pwd[0]);
            // 啶ŕ(s)̎@@Ȃ̂Œ
            *wd += 32;	if(zkc != 'h') zkc = 'K';

            gobi = wd+strlen(wd)-1;
            if(zkc != 'K' && strcmp(gobi, "s") == 0) *gobi = '\0';

	  }
	}
      }
 if(Dbg == 'Y') fprintf(dbg, "--- preConj --- wd = %s  zkc = %c  pwd[0] = %c \n", wd, zkc, pwd[0]);
      ct = -1;
//	caj[0][0] = '1';
	return;
    }
    // wd'-'܂ގ
    jh = instrc(wd, '-');
    if(jh >= 3 && jh <= strlen(wd)-2){
      // wd[jh-1]strlen(wd)-jh+1폜
      cutoff(wd,jh-1,strlen(wd)-jh+1);	strcpy(wdp,wd);
      zkc = Cmp = '~';	ct = -1;	return;
    }
    IRRG:
    irregVerb(&kakt);
    if(kakt == 'k') ct = -1;
}

irregVerb(kakt)
char *kakt;
{
    FILE *irreg;
    int  ln, sp;
    char irdata[55], wdd[WordLen], juk;
    char p1[20], *p2, og[20], *ogpt, *ptn, irregul[50];
    *kakt = 'k';
    if(zkc != 'j'){
      juk = 'n';	strcpy(wdd, wd);
    }
    else {
      juk = 'y';	ln = strlen(wd);
      for(sp = ln-1; sp > 0; sp--){
	if(wd[sp] == ' ') break;
      }
      right(wd, wdd, ln-1-sp);
    }

    if(strcmp(wdd, "am")==0 || strcmp(wdd, "are")==0 || strcmp(wdd, "is")==0){
      strcpy(wdd, "be");	zkc = 'v';	goto KAKT;
    }

    if(strcmp(wdd, "has") == 0){
      strcpy(wdd, "have");	zkc = 'H';	goto KAKT;
    }
    *kakt = 'n';
//    sprintf(irregul, "%sIRREGUL.HON", dir);
    strcpy(irregul, "IRREGUL.HON");
//    irreg = fopen("IRREGUL.HON", "r");
    irreg = fopen(irregul, "r");
    if(irreg == 0){ printf("file not exist\n");	goto KAKT; }
    while(fgets(irdata, 53, irreg)){
      if(ptn = (char*)index(irdata, ';')) *ptn = '\0';
      if(ogpt = (char*)index(irdata, ',')) ptcpy(og, irdata, ogpt);
      if(p2 = (char*)index(ogpt+2, ',')) ptcpy(p1, ogpt+1, p2++);
      else break;
      if(strcmp(p1, wdd) == 0){ zkc = 'p';
	strcpy(wdd, og);	*kakt = 'k';	break;
      }
      if(strcmp(p2, wdd) == 0){ zkc = 'P';
	strcpy(wdd, og);	*kakt = 'k';	break;
      }
    }
    fclose(irreg);
    KAKT:
    if(*kakt == 'k'){ caj[0][0] = '3';
      if(juk == 'n') strcpy(wd, wdd);
      else replaces(wd, sp+1, ln-1-sp, wdd);
    }
}

setsubiGo(){
    int  tg, ln, ln1;
    static char ly[4][5] = {"ily","ply","bly","ly"},
	   	le[4][5] = {"y","ple","ble","\0"};
    ln = strlen(wd)+1;
    for(tg = 0; tg <= 3; tg++){
      ln1 = strlen(ly[tg]);
      if(ln > ln1 && ninstr(ln-ln1, wd, ly[tg]) == ln-ln1){
	replaces(wd, ln-ln1-1, ln1, le[tg]);
	if(tg == 1 &&
	(strcmp(wd+ln-6, "arply")==0 || strcmp(wd+ln-6, "eelply")==0))
	  *(wd+ln-3) = '\0';
	zkc = 'l';
      }
    }
}
